# -*- coding: utf-8 -*-

import os
import sys
import requests
import base64

import shutil
import json
from pathlib import Path
from subprocess import check_output
import platform
import ctypes

from jarvis_sdk import jarvis_config
from jarvis_sdk import jarvis_auth


def special_processing_table_to_storage(config):

    # Process SQL file
    my_file = open(config["sql_file"], "r")
    temp = my_file.read()
    my_file.close()

    config['sql'] = str(base64.b64encode(bytes(temp, "utf-8")),"utf-8")
    #print("Encoded SQL data : " + str(data['sql']))
    del config["sql_file"]

    return config


def display_configuration_deploy_help(jarvis_configuration, firebase_user):

    try:

        # Get UID
        #
        uid = firebase_user["userId"]

        # Get default project
        #
        default_project = (jarvis_config.get_jarvis_configuration_file())["gcp_default_project"]

        url = jarvis_configuration["jarvis_api_endpoint"] + "configuration/help"
        data = {
            "payload": {
                "resource": "help",
                "uid" : uid,
                "gcp_project_id" : default_project
            }
        }
        headers = {
            "Content-type": "application/json",
            "Authorization": "Bearer " + firebase_user["idToken"]
        }

        r = requests.post(url, headers=headers, data=json.dumps(data))

        if r.status_code != 200:
            # print(r.headers)
            print("\nError : %s\n" % str(r.content, "utf-8"))
        else:
            response = r.json()
            print(response["payload"]["help"])

    except Exception as ex:
        print("Error while trying to contact Jarvis API ...")
        print(ex)
        return False

    return True


def deploy_configuration(input_conf_file, gcp_project_id, jarvis_configuration, firebase_user):

    # Check if the file exists
    #
    if os.path.isfile(input_conf_file) is False:
        print("File \"%s\" does not exists." % input_conf_file)
        return False

    # Read file and parse it as JSON
    #
    read_configuration = None
    try:
        with open(input_conf_file, "r") as f:
            read_configuration = json.load(f)
    except Exception as ex:
        print("Error while parsing configuration file.")
        return False

    # Check if we need special processing
    # gbq-to-gcs : needs to retrieve sql file
    #
    #
    try:
        if read_configuration["configuration_type"] == "gbq-to-gcs":
            read_configuration = special_processing_table_to_storage(read_configuration)
    except Exception as ex:
        print("Error while parsing configuration file for special processing.")
        return False


    # Call API
    #
    try:

        url = jarvis_configuration["jarvis_api_endpoint"] + "configuration"
        data = {
            "payload": {
                "resource": read_configuration,
                "gcp_project_id" : gcp_project_id,
                "uid" : firebase_user["userId"]
            }
        }
        headers = {
            "Content-type": "application/json",
            "Authorization": "Bearer " + firebase_user["idToken"
            ]}

        r = requests.put(url, headers=headers, data=json.dumps(data))

        if r.status_code != 200:
            print("\nError : %s\n" % str(r.content, "utf-8"))
            return False
        else:
            response = r.json()
            print(response["payload"]["message"])
            return True

    except Exception as ex:
        print("Error while trying to contact Jarvis API ...")
        print(ex)
        return False


def process_gcp_project_id(gcp_pid, default_pid):
    if gcp_pid is None:
        print("Using default GCP Project ID : %s" % default_pid)
        return default_pid
    else:
        print("Using provided GCP Project ID : %s" % gcp_pid)
        return gcp_pid


def process(args):

    print("Jarvis Configuration Manager.")

    # Get configuration
    #
    jarvis_configuration = jarvis_config.get_jarvis_configuration_file()

    # Get firebase user
    #
    firebase_user = jarvis_auth.get_refreshed_firebase_user(jarvis_configuration)

    # Process GCP project ID
    #
    gcp_project_id = process_gcp_project_id(args.gcp_project_id, jarvis_configuration["gcp_default_project"])

    if args.command == "deploy":
        if len(args.arguments) >= 2:
            if args.arguments[1] is not None:
                if args.arguments[1] == "help":
                    return display_configuration_deploy_help(jarvis_configuration, firebase_user)
                else:
                    return deploy_configuration(args.arguments[1],gcp_project_id, jarvis_configuration, firebase_user)
            else:
                print("Argument unknown." % args.arguments[1])
                return False
        else:
            return display_configuration_deploy_help(jarvis_configuration, firebase_user)

    return True
